# Copyright 2026 EcoFuture Technology Services LLC and contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Generated by Django 5.0.1 on 2024-10-20 00:05

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('entity', '0003_childentity__author_parent_selectors_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='childentity',
            old_name='_author_parent_selectors',
            new_name='autogen_author_parent_selectors',
        ),
        migrations.RenameField(
            model_name='childentity',
            old_name='_author_selectors',
            new_name='autogen_author_selectors',
        ),
        migrations.RenameField(
            model_name='childentity',
            old_name='_author_updated_selectors',
            new_name='autogen_author_updated_selectors',
        ),
        migrations.RenameField(
            model_name='dependententity',
            old_name='_author_parent_selectors',
            new_name='autogen_author_parent_selectors',
        ),
        migrations.RenameField(
            model_name='dependententity',
            old_name='_author_selectors',
            new_name='autogen_author_selectors',
        ),
        migrations.RenameField(
            model_name='dependententity',
            old_name='_author_updated_selectors',
            new_name='autogen_author_updated_selectors',
        ),
        migrations.RenameField(
            model_name='extendedentity',
            old_name='_author_parent_selectors',
            new_name='autogen_author_parent_selectors',
        ),
        migrations.RenameField(
            model_name='extendedentity',
            old_name='_author_selectors',
            new_name='autogen_author_selectors',
        ),
        migrations.RenameField(
            model_name='extendedentity',
            old_name='_author_updated_selectors',
            new_name='autogen_author_updated_selectors',
        ),
        migrations.RenameField(
            model_name='parententity',
            old_name='_author_selectors',
            new_name='autogen_author_selectors',
        ),
        migrations.RenameField(
            model_name='parententity',
            old_name='_author_updated_selectors',
            new_name='autogen_author_updated_selectors',
        ),
    ]
