# Copyright 2026 EcoFuture Technology Services LLC and contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Generated by Django 5.0.1 on 2025-12-23 13:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('permit', '0005_alter_grouppermission_slug_alter_permission_slug_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='grouppermission',
            old_name='name',
            new_name='name_en',
        ),
        migrations.RenameField(
            model_name='role',
            old_name='name',
            new_name='name_en',
        ),
        migrations.AddField(
            model_name='grouppermission',
            name='name_ru',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='Name'),
        ),
        migrations.AddField(
            model_name='role',
            name='name_ru',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='Name'),
        ),
    ]
