# Copyright 2026 EcoFuture Technology Services LLC and contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.apps import apps

from bazis.core.routes_abstract.jsonapi import JsonapiRouteBase


class RoleRoute(JsonapiRouteBase):
    """
    Defines the route for handling Role-related operations in the application,
    extending the JsonapiRouteBase to leverage JSON API standards. It specifies the
    model to be used and the actions that can be performed.
    """

    model = apps.get_model('permit.Role')
    actions = ['action_list', 'action_retrieve']
