# Copyright 2026 EcoFuture Technology Services LLC and contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Generated by Django 5.0.1 on 2024-10-24 09:04

import django.contrib.postgres.indexes
from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('entity', '0004_rename__author_parent_selectors_childentity_autogen_author_parent_selectors_and_more'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RemoveField(
            model_name='parententity',
            name='autogen_author_updated_selectors',
        ),
        migrations.AddIndex(
            model_name='childentity',
            index=django.contrib.postgres.indexes.GinIndex(fields=['autogen_author_selectors'], name='entity_chil_autogen_849c8d_gin'),
        ),
        migrations.AddIndex(
            model_name='childentity',
            index=django.contrib.postgres.indexes.GinIndex(fields=['autogen_author_updated_selectors'], name='entity_chil_autogen_c1a095_gin'),
        ),
        migrations.AddIndex(
            model_name='childentity',
            index=django.contrib.postgres.indexes.GinIndex(fields=['autogen_author_parent_selectors'], name='entity_chil_autogen_eec219_gin'),
        ),
        migrations.AddIndex(
            model_name='dependententity',
            index=django.contrib.postgres.indexes.GinIndex(fields=['autogen_author_selectors'], name='entity_depe_autogen_22be95_gin'),
        ),
        migrations.AddIndex(
            model_name='dependententity',
            index=django.contrib.postgres.indexes.GinIndex(fields=['autogen_author_updated_selectors'], name='entity_depe_autogen_c1c46a_gin'),
        ),
        migrations.AddIndex(
            model_name='dependententity',
            index=django.contrib.postgres.indexes.GinIndex(fields=['autogen_author_parent_selectors'], name='entity_depe_autogen_d06cc8_gin'),
        ),
        migrations.AddIndex(
            model_name='extendedentity',
            index=django.contrib.postgres.indexes.GinIndex(fields=['autogen_author_selectors'], name='entity_exte_autogen_55395d_gin'),
        ),
        migrations.AddIndex(
            model_name='extendedentity',
            index=django.contrib.postgres.indexes.GinIndex(fields=['autogen_author_updated_selectors'], name='entity_exte_autogen_3f6159_gin'),
        ),
        migrations.AddIndex(
            model_name='extendedentity',
            index=django.contrib.postgres.indexes.GinIndex(fields=['autogen_author_parent_selectors'], name='entity_exte_autogen_0a9cf8_gin'),
        ),
        migrations.AddIndex(
            model_name='parententity',
            index=django.contrib.postgres.indexes.GinIndex(fields=['autogen_author_selectors'], name='entity_pare_autogen_8f8317_gin'),
        ),
    ]
